
#' @title Helper function: transform integer to factor
#'
#' @description 
#' This function re-codes a factor with pre-specified factor levels, using an integer encoding as input.
#'
#' @param x an integer vector (that represents factor vector) to be transformed
#' @param lvls the original factor levels used
#' @return the same factor, now coded with the original levels
#'
#' @export
int2fact <- function(x, lvls) {
	nms <- names(lvls)
	for (i in 1:length(lvls)) {
		x[[nms[i]]] <-  lvls[[i]][x[[nms[i]]]]
	}
	x
}


#' @title Identity transformation
#'
#' @param x input
#' @return \code{x}
#'
#' @export
trans_id <- function(x) {
	x
}

#' @title 2 power x transformation
#'
#' @param x input
#' @return \code{2^x}
#'
#' @export
trans_2pow <- function(x) {
	2 ^ x
}

#' @title 2 power x transformation with round
#'
#' @param x input
#' @return \code{round(2^x)}
#'
#' @export
trans_2pow_round <- function(x) {
	round(2 ^ x)
}

#' @title 10 power x transformation
#'
#' @param x input
#' @return \code{10^x}
#'
#' @export
trans_10pow <- function(x) {
	10 ^ x
}


#' @title 10 power x transformation with round
#'
#' @param x input
#' @return \code{round(10^x)}
#'
#' @export
trans_10pow_round <- function(x) {
	round(10 ^ x)
}

