runAE <- function(
  batch_size=128L, # >0L
  lr=0.001,	# (0, 1)
  lrup=0.1,	# (0, 1)
  drp=0.1, # [0, 1)		
  num_layers=2, # T, F
  nfilter=60L, # >1L			
  activation="relu",  #"relu", "swish", "sigmoid"
  #padd="same", #"same", "causal"					
  seed=1
){
  tensorflow::set_random_seed(seed)
  set.seed(seed)
  
  source("functions.R")
  
  ## read data files
  data <- readRDS(file = "waterDataTraining.RDS")
  
  ## separate columns not directly passed to model
  data_label <- data$EVENT
  data_time <- data$Time
  data$Time <- NULL
  data$EVENT <- NULL
  
  ## differencing and moving average
  data <- ma_diff(data,5)
  
  ## replace NA values by moving average
  data <- na_locf(data)
  
  ## data split into training and validation
  train_index <- 1:round(nrow(data)*0.5)
  validation_index <- (round(nrow(data)*0.5)+1-64):nrow(data)
  train_data <- data[train_index,]
  validation_data <- data[validation_index,]
  train_data_labels <- data_label[train_index]
  validation_data_labels <- data_label[validation_index]
  
  ## mean and standard deviation for scaling
  scale_mean <- apply(train_data, 2, mean,na.rm=T)
  scale_sd <- apply(train_data, 2, sd,na.rm=T)
  
  ## scale data
  train_data_scaled <- scale(train_data, center = scale_mean, scale = scale_sd)
  validation_data_scaled <- scale(validation_data, center = scale_mean, scale = scale_sd)
  
  ## number of features in the data
  nfeats <- ncol(train_data_scaled) #number of features
  
  ## compute sequence blocks
  result <- reshape_sequences_lag_event(train_data_scaled,train_data_labels)
  x_seq_train <- result$x_seq
  x_seq_train_labels <- result$x_seq_labels
  x_seq_train_labels_exclude <- result$x_seq_labels_exclude
  result <- NULL
  
  ## exclude from training data: sequences that contain events 
  ## (we only train with )
  x_seq_train_noevent <- x_seq_train[!x_seq_train_labels_exclude,,]
  
  ## generate validation sequence blocks
  result <- reshape_sequences_lag_event(validation_data_scaled,validation_data_labels)
  x_seq_validation <- result$x_seq
  x_seq_validation_labels <- result$x_seq_labels
  x_seq_validation_labels_exclude <- result$x_seq_labels_exclude
  result <- NULL
  
  gc()
  
  k_clear_session()
  padd <- "same"
  strides <- 2
  kernelsize <- 6	
  ## Initialize model
  model = keras_model_sequential()
  ## First convolutional layer + dropout
  model	%>% layer_conv_1d(filters=nfilter, kernel_size = kernelsize,
                          input_shape=(c(dim(x_seq_train_noevent)[2], nfeats)),
                          padding=padd, strides=strides, activation=activation
  ) %>% layer_dropout(rate=drp)
  ## Further optional layers
  if(num_layers>1){
    for(i in 2:num_layers){
      model %>% layer_conv_1d(filters=nfilter, kernel_size = kernelsize,
                              padding=padd, strides=strides, activation=activation
      ) %>% layer_dropout(rate=drp)
    }
  }
  ## Further optional layers
  if(num_layers>1){
    for(i in 2:num_layers){
      model %>% layer_conv_1d_transpose(filters=nfilter, kernel_size = kernelsize,
                                        padding=padd, strides=strides, activation=activation
      ) %>% layer_dropout(rate=drp) 
    }
  }
  ## Last 'conventional' transposed convolutional layer
  model %>% layer_conv_1d_transpose(filters=nfilter, kernel_size = kernelsize,
                                    padding=padd, strides=strides, activation=activation) 
  ## Output layer (reshaping to output tensor)
  model %>% layer_conv_1d_transpose(filters=nfeats, kernel_size=1, padding=padd) 
  
  #print(summary(model))
  
  model %>% compile(optimizer = optimizer_adam(lr = lr), loss="mse")
  ##
  model %>% fit(x=x_seq_train_noevent,y=x_seq_train_noevent,
                epochs=20L,verbose=0L,batch_size=batch_size)
  
  ## Compute errors on training data (but now including the event sequences)
  train_pred <- model$predict(x_seq_train)
  train_err <- rowSums(abs(x_seq_train - train_pred))
  AUCtrain <- roc(x_seq_train_labels,train_err,direction="<")
  
  #train_pred_noevent <- model$predict(x_seq_train_noevent)
  #train_err_noevent <- apply(abs(x_seq_train_noevent - train_pred_noevent),1,sum)
  #thresh <- quantile(train_err_noevent,probs=0.999)
  
	## Find best threshold on training data
  thresh <- optimizeThreshold(x_seq_train_labels,train_err)
	
  ## Apply to validation data
  model %>% compile(optimizer = optimizer_adam(lr = lr*lrup), loss="mse")
  
  window <- 24*60
  validation_err <- NULL
  for(jj in seq(from=1,by=window,to=dim(x_seq_validation)[1])){
    print(jj)
    sel <- jj:min((jj+window-1),dim(x_seq_validation)[1])
    validation_predjj <- model$predict(x_seq_validation[sel,,])
    validation_errjj <- rowSums(abs(x_seq_validation[sel,,] - validation_predjj))
    validation_err <- c(validation_err,validation_errjj)
    outlying <- validation_errjj > thresh
    if(!all(outlying))
      model %>% fit(x=x_seq_validation[sel,,][!outlying,,,drop=F],y=x_seq_validation[sel,,][!outlying,,,drop=F],
                    epochs=2L,verbose=0L,batch_size=batch_size)
  }
  
  
  AUCvalidation <- roc(x_seq_validation_labels,validation_err,direction="<")	
  
  gc()
  k_clear_session()
  gc()
  
  res <- calculateScore(x_seq_validation_labels,validation_err>thresh)
  
  return(-c(as.numeric(AUCvalidation$auc),res$SCORE,as.numeric(AUCtrain$auc)))
}
