runLOCF_test <- function(
  seed=1L
){
  set.seed(seed)
  
  library("keras") #Neural Network Modeling
  library("imputeTS") #Imputation for preprocessing
  library("abind") #array binding
  library("pROC") #computing area under ROC curve 
  source("functions.R")
  
  ## read data files
  data <- readRDS(file = "waterDataTraining.RDS")
  datatest <- readRDS(file = "waterDataTestingUpload.RDS")
  data <- rbind(data,datatest)
  datatest <- NULL
  gc()
  
  
  ## separate columns not directly passed to model
  data_label <- data$EVENT
  data_time <- data$Time
  data$Time <- NULL
  data$EVENT <- NULL
  
  ## differencing and moving average
  data <- ma_diff(data,5)
  
  ## replace NA values by moving average
  data <- na_locf(data)
  
  ## data split into training and validation
  train_index <- 1:round(nrow(data)*0.5)
  validation_index <- (round(nrow(data)*0.5)+1-64):nrow(data) 
  train_data <- data[train_index,]
  validation_data <- data[validation_index,]
  train_data_labels <- data_label[train_index]
  validation_data_labels <- data_label[validation_index]
  
  
  ## mean and standard deviation for scaling
  scale_mean <- apply(train_data, 2, mean,na.rm=T)
  scale_sd <- apply(train_data, 2, sd,na.rm=T)
  
  ## scale data
  train_data_scaled <- scale(train_data, center = scale_mean, scale = scale_sd)
  validation_data_scaled <- scale(validation_data, center = scale_mean, scale = scale_sd)
  
  ## number of features in the data
  nfeats <- ncol(train_data_scaled) #number of features
  
  ## compute sequence blocks
  result <- reshape_sequences_lag_event(train_data_scaled,train_data_labels)
  x_seq_train <- result$x_seq
  x_seq_train_labels <- result$x_seq_labels
  x_seq_train_labels_exclude <- result$x_seq_labels_exclude
  result <- NULL
  
  ## subsample to vary training data set
  nseq <- dim(x_seq_train)[1]
  subsamplel <- sample(c(TRUE, FALSE),nseq,prob = c(0.66,0.33),replace=TRUE)  
  x_seq_train <- x_seq_train[subsamplel,,]
  x_seq_train_labels <- x_seq_train_labels[subsamplel]
  x_seq_train_labels_exclude <- x_seq_train_labels_exclude[subsamplel]
  
  ## exclude from training data: sequences that contain events 
  x_seq_train_noevent <- x_seq_train[!x_seq_train_labels_exclude,,]
  
  ## generate validation sequence blocks
  result <- reshape_sequences_lag_event(validation_data_scaled,validation_data_labels)
  x_seq_validation <- result$x_seq
  x_seq_validation_labels <- result$x_seq_labels
  x_seq_validation_labels_exclude <- result$x_seq_labels_exclude
  result <- NULL
	
  ## Compute 'errors' on training data 
  train_pred <- x_seq_train[-dim(x_seq_train)[1],,]
  train_err <- c(0,rowSums(abs(x_seq_train[-1,,] - train_pred)))
  AUCtrain <- roc(x_seq_train_labels,train_err,direction="<")
  
	## Find best threshold on training data
  thresh <- optimizeThreshold(x_seq_train_labels,train_err)
  
	## Apply to validation data
  validation_pred <- x_seq_validation[-dim(x_seq_validation)[1],,]
  validation_err <- c(0,rowSums(abs(x_seq_validation[-1,,] - validation_pred)))
  AUCvalidation <- roc(x_seq_validation_labels,validation_err,direction="<")	
  
  res <- calculateScore(x_seq_validation_labels,validation_err>thresh)
  
  return(-c(as.numeric(AUCvalidation$auc),res$SCORE,as.numeric(AUCtrain$auc)))
}
resTestLOCF <- sapply(1:20,runLOCF_test)
save(resTestLOCF,file="resultFileLOCF.RData")
