## If not installed, do:
# install.packages("reticulate")
# install.packages("dplyr")
# install.packages("callr")
# install.packages("plotly")
# install.packages("ggsci")
# install.packages("emoa")
# install.packages("imputeTS")
# install.packages("keras")
# install.packages("abind")
# install.packages("pROC")
# install.packages("isotree")
## Load the packages:
require(reticulate) 
require(dplyr) 
require(callr) 
require(plotly)
require(ggsci)
require(emoa)
library(imputeTS)
library(keras)
library(abind) 
library(pROC)

## Note, that this example requires python and the respective tensorflow / keras
## code to be set up correctly. 

## Note, that you have to make sure to use the right python
## version / environment (where the respective modules are installed)
## to execute the underlying run.py code,
## see e.g., 
#?reticulate::use_python

## Load a few R functions required for this example
source("getobjf.R")

source("misc.R")

source("plotting.R")

source("runIF.R")
source("runIF_test.R")

#source("runIF_test.R")

## configuration for the tuning problem
cfg <- list(
  ## Names of the parameters
  tunepars = c("sample_size","ntrees","ndim",   "ntry",   "max_depth","ncols_per_tree"),
  ## their lower bounds
  lower =    c(100,           10,       1,        1,        2,          2),
  ## their upper bounds
  upper =    c(500,         1000,   5,      10,      15,         100), 
  ## their type
  type =    c("integer",    "integer","integer","integer","integer",  "integer"),
  ## transformations to apply
  transformations = 
    c(        trans_id,      trans_id, trans_id, trans_id, trans_id,trans_id),
  ## another parameter that will not be tuned, but is fixed					 
  fixpars = list(),
  ## not used in this example (specify levels of categorical parameters):
  factorlevels = list(),
  ## not used in this example (specify parameters that are relative to other parameters)
  relpars = list(),
  run=runIF
)

## penalty value if modeling fails
penalty <- -c(0.5,0,0.5)

## create an objective function that contains the parameterized learning process
objf <- get_objf(cfg,
                 600, # timeout in seconds
                 penalty) # and the penalty

## create an objective function that contains the parameterized learning process
cfg_test <- cfg
cfg_test$run <- runIF_test
objf_test <- get_objf(cfg_test,
                      1200, # timeout in seconds
                      penalty) # and the penalty

## the following are some optional checks if everything works as expected
## (evaluating twice at the lower and upper bound)
#res11 <- objf(matrix(cfg$lower,1),1)
#res12 <- objf(matrix(cfg$lower,1),2)
#res21 <- objf(matrix(cfg$upper,1),1)
#res22 <- objf(matrix(cfg$upper,1),2)
#res11 <- objf_test(matrix(cfg$lower,1),1)
#res12 <- objf_test(matrix(cfg$lower,1),2)

## install SPOT:
#install.packages("SPOT")
## load SPOT package
require(SPOT)

resultfile <- "resultFileIF.RData" #TODO: currently testing 1:64 pattern

if(!file.exists(resultfile)){
  ## Run tuning with SPOT
  result <- spot(fun = objf,
                 lower=cfg$lower,
                 upper=cfg$upper,
                 control = list(types=cfg$type,
                                plots=T, # do progress plots
                                optimizer=optimDE, # optimize surrogate with DE
                                noise=TRUE, # problem is noisy
                                seedSPOT=1, # initial RNG seed for SPOT
                                seedFun=1, # initial, incremental seed for evaluation
                                designControl=list(size=50,replicates=1), # initial design
                                replicates=1, # evaluate each candidate 1 time
                                funEvals=150, # limit on number of evaluations
                                model=buildKriging, #Kriging / Gaussian process surogate
                                modelControl=list(target="ei",useLambda=T,reinterpolate=T), #Kriging configuration
                                optimizerControl=list(funEvals=400*length(cfg$lower)) # evaluation budget for DE
                 )
  )
  
  resultpp <- prepare_spot_result_plot(data=result,
                                       model=buildRandomForest,
                                       modelControl=list(types=cfg$type),
                                       namesx=cfg$tunepars,
                                       namesy="AUCvalidation",
                                       nameslog=c("F1validation","AUCtrain","duration")
  )
  resultpp2 <- prepare_spot_result_plot(data=result,
                                        model=buildKriging,
                                        modelControl=list(types=cfg$type,useLambda=T,reinterpolate=F),
                                        namesx=cfg$tunepars,
                                        namesy="AUCvalidation",
                                        nameslog=c("F1validation","AUCtrain","duration")
  )
  
  ## Test evaluation (unseen test data with resampled train+validation)
  resTest <- sapply(1:20,objf_test,x=resultpp$xbest)
  
  ## save result in RData file
  save(resultpp,resultpp2,result,resTest,
       file=resultfile)
}else{
  load(resultfile)
}
